/* vi:set ts=8 sts=0 sw=8:
 * $Id: multiple.c,v 1.6 2000/05/22 05:08:36 kahn Exp kahn $
 *
 * Copyright (C) 1998 Andy C. Kahn
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "win.h"
#include "main.h"
#include "misc.h"
#include "multiple.h"


enum Ef_mf_id_e {
	Ef_mf_dialog,
	Ef_mf_filename,		/* entry */
	Ef_mf_add,		/* button */
	Ef_mf_list,		/* list */
	Ef_mf_toolbar,		/* toolbar */
	Ef_mf_up,		/* button */
	Ef_mf_down,		/* button */
	Ef_mf_del,		/* button */
	Ef_mf_clear,		/* button */
	Ef_mf_close,		/* button */
	Ef_mf_MAX
};
typedef enum Ef_mf_id_e Ef_mf_id_t;

enum Ef_md_id_e {
	Ef_md_dialog,
	Ef_md_dirname,		/* entry */
	Ef_md_add,		/* button */
	Ef_md_list,		/* list */
	Ef_md_toolbar,		/* toolbar */
	Ef_md_up,		/* button */
	Ef_md_down,		/* button */
	Ef_md_del,		/* button */
	Ef_md_clear,		/* button */
	Ef_md_close,		/* button */
	Ef_md_MAX
};
typedef enum Ef_md_id_e Ef_md_id_t;


static void entry_mult_files_cb(GtkWidget *wgt, gpointer cbdata);
static void entry_mult_dirs_cb(GtkWidget *wgt, gpointer cbdata);
static void mult_files_close_cb(GtkWidget *wgt, gpointer cbdata);
static void mult_files_add_cb(GtkWidget *wgt, gpointer cbdata);
static void mult_dirs_close_cb(GtkWidget *wgt, gpointer cbdata);
static void mult_dirs_add_cb(GtkWidget *wgt, gpointer cbdata);


static ef_t ef_mf_widgets[] = {
	{
		"Multiple Files",
		Ef_mf_dialog,
		Ef_wgt_dialog
	},
	{
		"entry_mult_files",
		Ef_mf_filename,
		Ef_wgt_entry,
		GTK_SIGNAL_FUNC(entry_mult_files_cb),
	},
	{
		"but_mult_files_add",
		Ef_mf_add,
		Ef_wgt_button,
		GTK_SIGNAL_FUNC(mult_files_add_cb),
	},
	{
		"list_mult_files",
		Ef_mf_list,
		Ef_wgt_list,
	},
	{
		"tb_mult_files",
		Ef_mf_toolbar,
		Ef_wgt_toolbar,
	},
	{
		"but_mult_files_up",
		Ef_mf_up,
		Ef_wgt_button,
	},
	{
		"but_mult_files_down",
		Ef_mf_down,
		Ef_wgt_button,
	},
	{
		"but_mult_files_del",
		Ef_mf_del,
		Ef_wgt_button,
	},
	{
		"but_mult_files_clear",
		Ef_mf_clear,
		Ef_wgt_button,
		GTK_SIGNAL_FUNC(mult_files_clear_cb),
	},
	{
		"but_mult_files_close",
		Ef_mf_close,
		Ef_wgt_button,
		GTK_SIGNAL_FUNC(mult_files_close_cb),
	},
	{ NULL }
}; /* ef_mf_widgets[] */


static ef_t ef_md_widgets[] = {
	{
		"Multiple Directories",
		Ef_md_dialog,
		Ef_wgt_dialog
	},
	{
		"entry_mult_dirs",
		Ef_md_dirname,
		Ef_wgt_entry,
		GTK_SIGNAL_FUNC(entry_mult_dirs_cb),
	},
	{
		"but_mult_dirs_add",
		Ef_md_add,
		Ef_wgt_button,
		GTK_SIGNAL_FUNC(mult_dirs_add_cb),
	},
	{
		"list_mult_dirs",
		Ef_md_list,
		Ef_wgt_list,
	},
	{
		"tb_mult_dirs",
		Ef_md_toolbar,
		Ef_wgt_toolbar,
	},
	{
		"but_mult_dirs_up",
		Ef_md_up,
		Ef_wgt_button,
	},
	{
		"but_mult_dirs_down",
		Ef_md_down,
		Ef_wgt_button,
	},
	{
		"but_mult_dirs_del",
		Ef_md_del,
		Ef_wgt_button,
	},
	{
		"but_mult_dirs_clear",
		Ef_md_clear,
		Ef_wgt_button,
		GTK_SIGNAL_FUNC(mult_dirs_clear_cb),
	},
	{
		"but_mult_dirs_close",
		Ef_md_close,
		Ef_wgt_button,
		GTK_SIGNAL_FUNC(mult_dirs_close_cb),
	},
	{ NULL }
}; /* ef_md_widgets[] */


static int
list_items_add(win_t *w, int list_id, GSList *argvl, gboolean is_dir)
{
	GList *children, *childp;
	GtkWidget *list;
	GtkLabel *label;
	GtkListItem *litem;
	char *name, *buf;
	int num = 0, argc = 0;

	if ((is_dir && is_multi_dir_search(w)) || is_multi_file_search(w)) {
		list = (is_dir) ? w->dirwgts[list_id] : w->filewgts[list_id];
		g_assert(GTK_IS_LIST(list));
		g_assert(GTK_IS_CONTAINER(list));
		children = gtk_container_children(GTK_CONTAINER(list));
		if (!is_dir) {
			num = g_list_length(children);
			g_assert(num);
			argvl = g_slist_append(argvl, g_strdup("("));
			argc++;
		}

		for (childp = children; childp; childp = childp->next) {
			g_assert(GTK_IS_WIDGET(childp->data));
			g_assert(GTK_IS_LIST_ITEM(childp->data));
			litem = GTK_LIST_ITEM(childp->data);
			g_assert(GTK_IS_LABEL(GTK_BIN(litem)->child));
			label = GTK_LABEL(GTK_BIN(litem)->child);
			gtk_label_get(label, &name);
			g_assert(name);
			if (is_dir) {
				argvl = g_slist_append(argvl, g_strdup(name));
				argc++;
			} else {
				if (w->findopts & FO_CASE_SENS)
					argvl = g_slist_append(argvl,
							g_strdup("-name"));
				else
					argvl = g_slist_append(argvl,
							g_strdup("-iname"));
				argc++;

				/* exact match, or substring match */
				if (w->findopts & FO_PARTIAL_FNAME) {
					buf = g_strdup_printf("*%s*", name);
					argvl = g_slist_append(argvl, buf);
				} else {
					argvl = g_slist_append(argvl,
							       g_strdup(name));
				}
				argc++;

				num--;
				if (num > 0) {
					argvl = g_slist_append(argvl,
							       g_strdup("-o"));
					argc++;
				} else {
					g_assert(childp->next == NULL);
				}
			}
		}

		if (!is_dir) {
			argvl = g_slist_append(argvl, g_strdup(")"));
			argc++;
		}
	}

	return argc;
} /* list_items_add */


static gboolean
is_multi_common(GtkWidget **wgts, int list_id)
{
	GtkWidget *list;
	GList *children;

	if (!wgts)
		return FALSE;

	list = wgts[list_id];
	g_assert(GTK_IS_LIST(list));
	if ((children = gtk_container_children(GTK_CONTAINER(list))))
		return TRUE;

	return FALSE;
} /* is_multi_common */


gboolean
is_multi_file_search(win_t *w)
{
	return is_multi_common(w->filewgts, Ef_mf_list);
} /* is_multi_file_search */


gboolean
is_multi_dir_search(win_t *w)
{
	return is_multi_common(w->dirwgts, Ef_md_list);
} /* is_multi_dir_search */


int
multi_dir_add(win_t *w, GSList *argvl)
{
	return list_items_add(w, Ef_md_list, argvl, TRUE);
} /* multi_dir_add */


int
multi_file_add(win_t *w, GSList *argvl)
{
	return list_items_add(w, Ef_mf_list, argvl, FALSE);
} /* multi_file_add */


static int
list_menu_popup_cb(GtkWidget *wgt, GdkEventButton *event, gpointer cbdata)
{
	return FALSE;
} /* list_menu_popup_cb */


static void
mult_dialog_create(win_t *w, gboolean is_dir)
{
	GtkWidget **wgts, *list;
	GladeXML *xml;
	ef_t *efp, *efwgts;
	int num, top_id, list_id;

	g_assert(w);
	wgts   = (is_dir) ? w->dirwgts    : w->filewgts;
	top_id = (is_dir) ? Ef_md_dialog  : Ef_mf_dialog;

	if (wgts) {
		gtk_widget_show(wgts[top_id]);
		return;
	}

	if (is_dir) {
		num = (sizeof(ef_md_widgets) / sizeof(efwgts[0])) - 1;
		efwgts = ef_md_widgets;
	} else {
		num = (sizeof(ef_mf_widgets) / sizeof(efwgts[0])) - 1;
		efwgts = ef_mf_widgets;
	}
	wgts = g_new0(GtkWidget *, num);
	if (is_dir)
		w->dirwgts = wgts;
	else
		w->filewgts = wgts;

	xml = my_glade_xml_get(is_dir ?
			       "Multiple Directories" : "Multiple Files");
	g_assert(xml);
#ifdef ENABLE_DEBUG
	g_print("mult_dialog_create: num = %d (max = %d)\n",
		num, (is_dir) ? Ef_md_MAX : Ef_mf_MAX);
	num = 0;
#endif
	for (efp = efwgts; efp->glname; efp++) {
		win_widgets_setup(xml, w, wgts, efp);
#ifdef ENABLE_DEBUG
		g_assert(efp->wgt_id == num);
		num++;
#endif
	}
	gtk_object_destroy(GTK_OBJECT(xml));
	gtk_window_set_position(GTK_WINDOW(wgts[top_id]), GTK_WIN_POS_MOUSE);
	gtk_window_set_transient_for(GTK_WINDOW(wgts[top_id]),
				     GTK_WINDOW(win_main_toplev(w)));

	/* gtk_clist_set_reorderable */
	list_id = (is_dir) ? Ef_md_list: Ef_mf_list;
	list = wgts[list_id];
	gtk_signal_connect(GTK_OBJECT(list), "button_press_event",
			   GTK_SIGNAL_FUNC(list_menu_popup_cb), w);

	gtk_widget_show(wgts[top_id]);
} /* mult_dialog_create */


void
mult_files_cb(GtkWidget *wgt, gpointer cbdata)
{
	mult_dialog_create(cbdata, FALSE);
} /* mult_files_cb */


void
mult_dirs_cb(GtkWidget *wgt, gpointer cbdata)
{
	mult_dialog_create(cbdata, TRUE);
} /* mult_dirs_cb */


static void
entry_mult_common_cb(win_t *w, GtkWidget **wgts, int entry_id, int add_id)
{
	char *txt;

	g_assert(GTK_IS_ENTRY(wgts[entry_id]));
	txt = gtk_entry_get_text(GTK_ENTRY(wgts[entry_id]));
	gtk_widget_set_sensitive(wgts[add_id], (txt && strlen(txt) > 0));
} /* entry_mult_cb */


static void
entry_mult_files_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	entry_mult_common_cb(w, w->filewgts, Ef_mf_filename, Ef_mf_add);
} /* entry_mult_files_cb */


static void
entry_mult_dirs_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	entry_mult_common_cb(w, w->dirwgts, Ef_md_dirname, Ef_md_add);
} /* entry_mult_dirs_cb */


static void
mult_clear_common(win_t *w, GtkWidget **wgts, int list_id, gboolean is_dir)
{
	GtkWidget *list;
	GList *children;

	if (!wgts)	/* can be NULL if only one out of the two */
		return; /* (multi-file or multi-dir) dialogs was used */
	list = wgts[list_id];
	children = gtk_container_children(GTK_CONTAINER(list));
	gtk_list_remove_items(GTK_LIST(list), children);
	children = gtk_container_children(GTK_CONTAINER(list));
	g_assert(children == NULL);
	win_fd_entry_setclear(w, TRUE, is_dir);
	win_fd_entry_sensitive(w, TRUE, is_dir);
} /* mult_clear_common */


void
mult_files_clear_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	mult_clear_common(w, w->filewgts, Ef_mf_list, FALSE);
} /* mult_files_clear_cb */


static void
mult_files_close_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	gtk_widget_hide(w->filewgts[Ef_mf_dialog]);
} /* mult_files_close_cb */


static void
mult_add_common(win_t *w, GtkWidget **wgts, int entry_id, int list_id,
		gboolean is_dir)
{
	GtkWidget *list, *litem;
	char *txt;

	g_assert(GTK_IS_ENTRY(wgts[entry_id]));
	txt = gtk_entry_get_text(GTK_ENTRY(wgts[entry_id]));
	g_assert(txt);
	g_assert(strlen(txt) > 0);
	list = wgts[list_id];
	g_assert(GTK_IS_LIST(list));
	litem = gtk_list_item_new_with_label(txt);
	gtk_container_add(GTK_CONTAINER(list), litem);
	gtk_editable_delete_text(GTK_EDITABLE(wgts[entry_id]), 0, strlen(txt));
	gtk_widget_show(litem);
	win_fd_entry_setclear(w, FALSE, is_dir);
	win_fd_entry_sensitive(w, FALSE, is_dir);
} /* mult_add_common */


static void
mult_files_add_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	mult_add_common(w, w->filewgts, Ef_mf_filename, Ef_mf_list, FALSE);
} /* mult_files_add_cb */


void
mult_dirs_clear_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	mult_clear_common(w, w->dirwgts, Ef_md_list, TRUE);
} /* mult_dirs_clear_cb */


static void
mult_dirs_close_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	gtk_widget_hide(w->dirwgts[Ef_md_dialog]);
} /* mult_dirs_close_cb */


static void
mult_dirs_add_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	mult_add_common(w, w->dirwgts, Ef_md_dirname, Ef_md_list, TRUE);
} /* mult_dirs_add_cb */


/* the end */
