/* vi:set ts=8 sts=0 sw=8:
 * $Id: win.c,v 1.24 2000/05/20 23:53:14 kahn Exp kahn $
 *
 * Copyright (C) 1998 Andy C. Kahn
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <config.h>
#include <string.h>
#include "win.h"
#include "main.h"
#include "misc.h"
#include "msgbar.h"
#include "results.h"
#include "advopts.h"
#include "multiple.h"


enum ef_main_id_e {
	Ef_main_toplev,
	Ef_main_msgbar,		/* label */
	Ef_main_file_new,	/* menuitem */
	Ef_main_quit,		/* menuitem */
	Ef_main_win_new,	/* menuitem */
	Ef_main_win_close,	/* menuitem */
	Ef_main_search_prev,	/* menuitem */
	Ef_main_about,		/* menuitem */
	Ef_main_adv_search,	/* checkbutton */
	Ef_main_show_adv_opts,	/* button */
	Ef_main_frame_params,	/* frame */

	Ef_main_toolbar,
	Ef_main_clear,		/* button */
	Ef_main_search,		/* button */
	Ef_main_stop,		/* button */
	Ef_main_prefs,		/* button */
	Ef_main_close,		/* button */

	Ef_main_find_text,	/* entry */
	Ef_main_find_combo,	/* gnome entry */
	Ef_main_find_tb,	/* toolbar */
	Ef_main_mult_files,	/* button */
	Ef_main_dir_text,	/* entry */
	Ef_main_dir_combo,	/* gnome file entry */
	Ef_main_dir_tb,		/* toolbar */
	Ef_main_mult_dirs,	/* button */
	Ef_main_search_subdir,	/* checkbutton */
	Ef_main_partial_fname,	/* checkbutton */
	Ef_main_case_sens,	/* checkbutton */

	Ef_main_MAX
};
typedef enum ef_main_id_e ef_main_id_t;

static void about_cb(GtkWidget *wgt, gpointer cbdata);
static void search_clear_cb(GtkWidget *wgt, gpointer cbdata);
static void search_prev_cb(GtkWidget *wgt, gpointer cbdata);
static void search_begin_cb(GtkWidget *wgt, gpointer cbdata);
static void search_stop_cb(GtkWidget *wgt, gpointer cbdata);
static void search_entry_file_dir_cb(GtkWidget *wgt, gpointer cbdata);
static void search_subdir_cb(GtkWidget *wgt, gpointer cbdata);
static void search_partial_fname_cb(GtkWidget *wgt, gpointer cbdata);
static void search_case_sens_cb(GtkWidget *wgt, gpointer cbdata);
static void search_adv_cb(GtkWidget *wgt, gpointer cbdata);
static win_t *win_main_widgets_setup(GladeXML *xml);
static int win_main_del_event_cb(GtkWidget *, GdkEvent *ev, gpointer cbdata);

static char *cb_signal_names[] = {
	"delete_event",	/* Ef_wgt_toplev */
	NULL,		/* Ef_wgt_dialog */
	"clicked",	/* Ef_wgt_button */
	"clicked",	/* Ef_wgt_checkbutton */
	NULL,		/* Ef_wgt_radiobutton */
	"changed",	/* Ef_wgt_entry */
	NULL,		/* Ef_wgt_combo */
	NULL,		/* Ef_wgt_clist */
	NULL,		/* Ef_wgt_list */
	NULL,		/* Ef_wgt_text */
	NULL,		/* Ef_wgt_vbox */
	NULL,		/* Ef_wgt_table */
	NULL,		/* Ef_wgt_frame */
	NULL,		/* Ef_wgt_notebook */
	"changed",	/* Ef_wgt_spinbutton */
	NULL,		/* Ef_wgt_optmenu */
	NULL,		/* Ef_wgt_toolbar */
	NULL,		/* Ef_wgt_label */
	"activate",	/* Ef_wgt_menuitem */
	NULL,		/* Ef_wgt_dont_care */
};

static ef_t ef_main_widgets[] = {
	{
		PACKAGE,
		Ef_main_toplev,
		Ef_wgt_toplev,
		GTK_SIGNAL_FUNC(win_main_del_event_cb)
	},
	{
		"msgbar",
		Ef_main_msgbar,		/* don't know what it is, so just */
		Ef_wgt_dont_care,	/* ignore it */
	},
	{
		"menuitem_search_new",
		Ef_main_file_new,
		Ef_wgt_menuitem,
		GTK_SIGNAL_FUNC(search_clear_cb)
	},
	{
		"menuitem_quit",
		Ef_main_quit,
		Ef_wgt_menuitem,
		GTK_SIGNAL_FUNC(ef_quit_cb)
	},
	{
		"menuitem_win_new",
		Ef_main_win_new,
		Ef_wgt_menuitem,
		GTK_SIGNAL_FUNC(win_new_cb)
	},
	{
		"menuitem_win_close",
		Ef_main_win_close,
		Ef_wgt_menuitem,
		GTK_SIGNAL_FUNC(win_close_cb)
	},
	{
		"menuitem_search_prev",
		Ef_main_search_prev,
		Ef_wgt_menuitem,
		GTK_SIGNAL_FUNC(search_prev_cb)
	},
	{
		"menuitem_about",
		Ef_main_about,
		Ef_wgt_menuitem,
		GTK_SIGNAL_FUNC(about_cb)
	},
	{
		"chkbut_adv_search",
		Ef_main_adv_search,
		Ef_wgt_checkbutton,
		GTK_SIGNAL_FUNC(search_adv_cb)
	},
	{
		"but_show_opts",
		Ef_main_show_adv_opts,
		Ef_wgt_button,
		GTK_SIGNAL_FUNC(advopts_cb)
	},
	{
		"frame_params",
		Ef_main_frame_params,
		Ef_wgt_frame
	},

	/* main toolbar */
	{
		"main_toolbar",
		Ef_main_toolbar,
		Ef_wgt_toolbar
	},
	{
		"tb_clear1",
		Ef_main_clear,
		Ef_wgt_button,
		GTK_SIGNAL_FUNC(search_clear_cb)
	},
	{
		"tb_search1",
		Ef_main_search,
		Ef_wgt_button,
		GTK_SIGNAL_FUNC(search_begin_cb)
	},
	{
		"tb_stop1",
		Ef_main_stop,
		Ef_wgt_button,
		GTK_SIGNAL_FUNC(search_stop_cb)
	},
	{
		"tb_prefs1",
		Ef_main_prefs,
		Ef_wgt_button,
	},
	{
		"tb_close1",
		Ef_main_close,
		Ef_wgt_button,
		GTK_SIGNAL_FUNC(win_close_cb)
	},

	/* "Basic" search page */
	{
		"entry_find",
		Ef_main_find_text,
		Ef_wgt_entry,
		GTK_SIGNAL_FUNC(search_entry_file_dir_cb)
	},
	{
		"gnoentry_find",
		Ef_main_find_combo,
		Ef_wgt_dont_care,
	},
	{
		"tb_mult_files",
		Ef_main_find_tb,
		Ef_wgt_toolbar,
	},
	{
		"but_mult_files",
		Ef_main_mult_files,
		Ef_wgt_button,
		GTK_SIGNAL_FUNC(mult_files_cb)
	},
	{
		"entry_directory",
		Ef_main_dir_text,
		Ef_wgt_entry,
		GTK_SIGNAL_FUNC(search_entry_file_dir_cb)
	},
	{
		"fileentry_directory",
		Ef_main_dir_combo,
		Ef_wgt_dont_care,
	},
	{
		"tb_mult_dirs",
		Ef_main_dir_tb,
		Ef_wgt_toolbar,
	},
	{
		"but_mult_dirs",
		Ef_main_mult_dirs,
		Ef_wgt_button,
		GTK_SIGNAL_FUNC(mult_dirs_cb)
	},
	{
		"chkbut_search_subdir",
		Ef_main_search_subdir,
		Ef_wgt_checkbutton,
		GTK_SIGNAL_FUNC(search_subdir_cb)
	},
	{
		"chkbut_partial_fname",
		Ef_main_partial_fname,
		Ef_wgt_checkbutton,
		GTK_SIGNAL_FUNC(search_partial_fname_cb)
	},
	{
		"chkbut_case_sens",
		Ef_main_case_sens,
		Ef_wgt_checkbutton,
		GTK_SIGNAL_FUNC(search_case_sens_cb)
	},
	{ NULL }
}; /* ef_main_widgets[] */


/*
 * "About" menuitem callback.
 */
static void
about_cb(GtkWidget *wgt, gpointer cbdata)
{
	GladeXML *xml;
	GtkWidget *dlg;
	win_t *w = (win_t *)cbdata;

	xml = my_glade_xml_get("About");
	g_assert(xml);
	dlg = my_glade_widget_get(xml, "About", GLADE_MODAL);
	gnome_dialog_set_parent(GNOME_DIALOG(dlg),
				GTK_WINDOW(win_main_toplev(w)));
	gtk_object_destroy(GTK_OBJECT(xml));
	gtk_widget_show(dlg);
} /* about_cb */


/*
 * Callback for the "clear" button on the toolbar.  For a basic search, simply
 * clears the text entry fields for the filename and directory name.  For an
 * advanced search, also clears the list of multiple files and directories by
 * calling mult_files_clear_cb() and mult_dirs_clear_cb().
 */
static void
search_clear_cb(GtkWidget *wgt, gpointer cbdata)
{
	ef_main_id_t id;
	GtkWidget *entry;
	char *txt;
	win_t *w = (win_t *)cbdata;

	for (id = Ef_main_find_text; ; id = Ef_main_dir_text) {
		entry = w->mainwgts[id];
		g_assert(GTK_IS_ENTRY(entry));
		if (GTK_WIDGET_IS_SENSITIVE(entry)) {
			txt = gtk_entry_get_text(GTK_ENTRY(entry));
			gtk_editable_delete_text(GTK_EDITABLE(entry),
						 0, strlen(txt));
		} else {
			mult_files_clear_cb(NULL, w);
			mult_dirs_clear_cb(NULL, w);
		}
		if (id == Ef_main_dir_text)
			break;
	}
} /* search_clear_cb */


/*
 * Frees the predicate linked list.  Called by search_reset().
 */
static void
search_reset_pred_free(struct predicate *pred)
{
	struct predicate *pp, *np;

	pp = pred;
	while (pp) {
		np = pp->pred_next;
		pp->pred_next = NULL;
		g_free(pp);
		pp = np;
	}
} /* search_reset_pred_free */


/*
 * Frees a uid linked list.  Called by search_reset().
 */
static void
search_reset_uid_list_free(struct userid **uilist)
{
	struct userid *uip, *next;

	uip = *uilist;
	while (uip) {
		next = uip->next;
		g_free(uip->name);
		g_free(uip);
		uip = next;
	}

	*uilist = NULL;

} /* search_reset_uid_list_free */


/*
 * Sets or clears an FO_* flag.
 */
static void
findopts_set_clear(win_t *w, findopt_t opt, gboolean set)
{
	if (set)
		w->findopts |= opt;
	else
		w->findopts &= ~opt;
} /* findopts_set_clear */


/*
 * Resets the window structure to reflect a "new" search.
 */
static void
search_reset(win_t *w)
{
	GtkWidget *wgt;

	/* make sure widget is still there */
	if (w->progbar && GTK_IS_WIDGET(w->progbar)) {
		gtk_progress_set_activity_mode(w->progbar, FALSE);
		gtk_progress_set_value(w->progbar, 0);
	}

	/* keep (reuse) previously selection options on main window */
	if ((wgt = w->mainwgts[Ef_main_search_subdir]) &&
	    GTK_IS_WIDGET(wgt) && GTK_WIDGET_IS_SENSITIVE(wgt)) {
		findopts_set_clear(w, FO_SEARCH_SUBDIR,
				   GTK_TOGGLE_BUTTON(wgt)->active);
	}
	if ((wgt = w->mainwgts[Ef_main_partial_fname]) && GTK_IS_WIDGET(wgt)) {
		findopts_set_clear(w, FO_PARTIAL_FNAME,
				   GTK_TOGGLE_BUTTON(wgt)->active);
	}
	if ((wgt = w->mainwgts[Ef_main_case_sens]) && GTK_IS_WIDGET(wgt)) {
		findopts_set_clear(w, FO_CASE_SENS,
				   GTK_TOGGLE_BUTTON(wgt)->active);
	}

	w->stop_search = FALSE;
	w->is_searching = FALSE;
	w->num_found = 0;
	w->num_errs = 0;

	/* set default values for stuff that used to be static/global */
	search_reset_uid_list_free(&(w->user_alist));
	search_reset_uid_list_free(&(w->nouser_alist));
	search_reset_uid_list_free(&(w->group_alist));
	search_reset_uid_list_free(&(w->nogroup_alist));

	w->kilobytes = -1;

	w->launch_first_time = 1;
	w->fstype_known = FALSE;
	g_free(w->cur_fstype);
	w->cur_fstype = NULL;
	w->cur_dev = 0;

	search_reset_pred_free(w->predicates);
	w->predicates = NULL;
#if 0
	/* the contents pointed at by these should be freed in the previous
	 * call to search_reset_pred_free(), but i'm keeping these two lines
	 * here as a placeholder/reminder. */
	g_free(w->last_pred);
	g_free(w->eval_tree);
#endif
	w->last_pred = NULL;
	w->eval_tree = NULL;
	w->do_dir_first = TRUE;
	w->maxdepth = -1;
	w->mindepth = -1;
	w->cur_day_start = time ((time_t *) 0) - DAYSECS;
	w->full_days = FALSE;
	w->no_leaf_check = FALSE;
	w->stay_on_filesystem = FALSE;
#ifndef HAVE_FCHDIR
	g_free(w->starting_dir);
	w->starting_dir = NULL;
#else
	w->starting_desc = 0;
#endif
	w->have_stat = FALSE;
	w->exit_status = 0;
	w->dereference = FALSE;
	g_free(w->dir_ids);
	w->dir_ids = NULL;
	w->dir_alloc = 0;
	w->dir_curr = -1;
	w->root_dev = 0;
#ifdef CACHE_IDS
	g_free(uid_unused);
	w->uid_unused = NULL;
	w->uid_allocated = 0;
	g_free(gid_unused);
	w->gid_unused = NULL;
	w->gid_allocated = 0;
#endif
	(void)msgbar_clear(w);
} /* search_reset */


/*
 * Starts the search.
 */
static void
search_exec(win_t *w, int argc, char *argv[])
{
	extern int main2(win_t *w, int argc, char *argv[]);
	int ret;
#ifdef ENABLE_DEBUG
	int i;
	printf("argc = %d\n", argc);
	for (i = 0; i < argc; i++)
		printf("argv[%d] = '%s'\n", i, argv[i]);
#endif
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_stop], TRUE);
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_frame_params], FALSE);
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_search], FALSE);
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_clear], FALSE);
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_file_new], FALSE);

	gtk_progress_set_activity_mode(w->progbar, TRUE);
	w->is_searching = TRUE;
	ret = main2(w, argc, argv);
	w->is_searching = FALSE;

	if (w->close_window) {
		win_close_cb(NULL, w);
		return;
	}

	gtk_widget_set_sensitive(w->mainwgts[Ef_main_file_new], TRUE);
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_clear], TRUE);
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_search], TRUE);
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_frame_params], TRUE);
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_stop], FALSE);

	results_found_set(w);
	results_dialog_show(w);
} /* search_exec */


/*
 * Show results from last search
 */
static void
search_prev_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	results_prev_show(w);
} /* search_prev_cb */


/*
 * Minor cleanup after a search has finished.
 */
static void
search_cleanup(int argc, char **argv, GSList *argvl)
{
	int i;

	for (i = 0; i < argc; i++)
		g_free(argv[i]);
	g_free(argv);
	g_slist_free(argvl);
} /* search_cleanup */


/*
 * Common code to start executing a search.
 */
static void
search_exec_common(win_t *w, int argc, GSList *argvl)
{
	GSList *avlp;
	char **argv;
	int i;

	/* Setup command line style argc and argv. */
	argv = g_new(char *, argc + 1);
	for (i = 0, avlp = argvl; i < argc && avlp; i++, avlp = avlp->next)
		argv[i] = avlp->data;
	argv[argc] = NULL;

	results_widgets_setup(w);
	search_exec(w, argc, argv);
	search_reset(w);
	search_cleanup(argc, argv, argvl);
} /* search_exec_common */


/*
 * Adds director(ies) to the search query.
 */
static int
search_dir_add(win_t *w, GSList *argvl)
{
	char *buf, *txt;

	if (is_multi_dir_search(w))
		return multi_dir_add(w, argvl);

	txt = gtk_entry_get_text(GTK_ENTRY(w->mainwgts[Ef_main_dir_text]));
	g_assert(txt);
	buf = g_strdup_printf("%s", txt);
	argvl = g_slist_append(argvl, buf);

	return 1;
} /* search_dir_add */


/*
 * Adds file(s) to the search query.
 */
static int
search_file_add(win_t *w, GSList *argvl)
{
	char *buf, *txt;

	if (is_multi_file_search(w))
		return multi_file_add(w, argvl);

	/* indicate what to search for */
	if (w->findopts & FO_CASE_SENS)
		argvl = g_slist_append(argvl, g_strdup("-name"));
	else
		argvl = g_slist_append(argvl, g_strdup("-iname"));

	/* exact match, or substring match */
	txt = gtk_entry_get_text( GTK_ENTRY(w->mainwgts[Ef_main_find_text]));
	g_assert(txt);
	if (w->findopts & FO_PARTIAL_FNAME) {
		buf = g_strdup_printf("*%s*", txt);
		argvl = g_slist_append(argvl, buf);
	} else {
		argvl = g_slist_append(argvl, g_strdup(txt));
	}

	return 2;
} /* search_file_add */


/*
 * Starts a "basic" search.
 */
static void
search_basic_begin(win_t *w)
{
	GSList *argvl;
	int argc;

	argvl = g_slist_append(NULL, g_strdup("find"));
	argc = 1;

	/* append directory to search */
	argc += search_dir_add(w, argvl);

	/* search subdirectories? */
	if ((w->findopts & FO_SEARCH_SUBDIR) == 0) {
		argvl = g_slist_append(argvl, g_strdup("-maxdepth"));
		argvl = g_slist_append(argvl, g_strdup("1"));
		argc += 2;
	}

	/* add file(s) to search for */
	argc += search_file_add(w, argvl);

	/* where to print */
	argvl = g_slist_append(argvl, g_strdup("-wprint"));
	argc++;

	g_assert(argc == g_slist_length(argvl));
	search_exec_common(w, argc, argvl);
} /* search_basic_begin */


/*
 * Starts an "advanced" search.
 */
static void
search_adv_begin(win_t *w)
{
	GSList *argvl;
	int argc;

	argvl = g_slist_append(NULL, g_strdup("find"));
	argc = 1;

	/* append directory to search */
	argc += search_dir_add(w, argvl);

	/* insert "Options" */
	argc += advopts_options_add(w, argvl);

	/* add file(s) to search for */
	argc += search_file_add(w, argvl);

	/* insert "Tests" */
	argc += advopts_tests_add(w, argvl);

	/* insert "Actions" */
	argc += advopts_actions_add(w, argvl);

	g_assert(argc == g_slist_length(argvl));
	search_exec_common(w, argc, argvl);
} /* search_adv_begin */


/*
 * Callback for "start search" button on toolbar.
 */
static void
search_begin_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	g_assert(w->mainwgts[Ef_main_find_text]);
	g_assert(w->mainwgts[Ef_main_dir_text]);

	if (GTK_TOGGLE_BUTTON(w->mainwgts[Ef_main_adv_search])->active)
		search_adv_begin(w);
	else
		search_basic_begin(w);
} /* search_begin_cb */


/*
 * Callback for "stop search" button on toolbar.
 */
static void
search_stop_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	w->stop_search = TRUE;
} /* search_stop_cb */


/*
 * Callback for "changed" signal for either text entry field.
 */
static void
search_entry_file_dir_cb(GtkWidget *wgt, gpointer cbdata)
{
	char *findtxt, *dirtxt;
	win_t *w = (win_t *)cbdata;
	int len1 = 0, len2 = 0;
	gboolean clear_sens = FALSE, search_sens = FALSE;

	g_assert(GTK_IS_ENTRY(w->mainwgts[Ef_main_find_text]));
	findtxt = gtk_entry_get_text(GTK_ENTRY(w->mainwgts[Ef_main_find_text]));
	if (findtxt)
		len1 = strlen(findtxt);

	g_assert(GTK_IS_ENTRY(w->mainwgts[Ef_main_dir_text]));
	dirtxt = gtk_entry_get_text(GTK_ENTRY(w->mainwgts[Ef_main_dir_text]));
	if (dirtxt)
		len2 = strlen(dirtxt);


	if (len1 || len2)
		clear_sens = TRUE;

	if (len1 && len2)
		search_sens = TRUE;

	gtk_widget_set_sensitive(w->mainwgts[Ef_main_clear], clear_sens);
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_file_new], clear_sens);
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_search], search_sens);
} /* search_entry_file_dir_cb */


/*
 * Callback for "search subdirectories" checkbutton.
 */
static void
search_subdir_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	g_assert(GTK_IS_CHECK_BUTTON(wgt));
	findopts_set_clear(w, FO_SEARCH_SUBDIR, GTK_TOGGLE_BUTTON(wgt)->active);
} /* search_subdir_cb */


/*
 * Callback for "partial filenames" checkbutton.
 */
static void
search_partial_fname_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	g_assert(GTK_IS_CHECK_BUTTON(wgt));
	findopts_set_clear(w, FO_PARTIAL_FNAME, GTK_TOGGLE_BUTTON(wgt)->active);
} /* search_partial_fname_cb */


/*
 * Callback for "case sensitive" checkbutton.
 */
static void
search_case_sens_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	g_assert(GTK_IS_CHECK_BUTTON(wgt));
	findopts_set_clear(w, FO_CASE_SENS, GTK_TOGGLE_BUTTON(wgt)->active);
} /* search_case_sens_cb */


/*
 * Callback for "Advanced search" checkbutton.
 */
static void
search_adv_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w = (win_t *)cbdata;

	g_assert(GTK_IS_CHECK_BUTTON(wgt));
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_show_adv_opts],
				 GTK_TOGGLE_BUTTON(wgt)->active);
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_search_subdir],
				 !GTK_TOGGLE_BUTTON(wgt)->active);

#if 0
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_partial_fname],
				 !GTK_TOGGLE_BUTTON(wgt)->active);
	gtk_widget_set_sensitive(w->mainwgts[Ef_main_case_sens],
				 !GTK_TOGGLE_BUTTON(wgt)->active);
#endif
} /* search_adv_cb */


/*
 * Returns TRUE if window was closed.  FALSE if close is delayed.
 */
static gboolean
win_close_common(win_t *w)
{
	if (w->is_searching) {
		w->close_window = TRUE;
		w->stop_search = TRUE;
		return FALSE;
	}

	results_dialog_destroy(w);
	search_reset(w);
	gtk_widget_destroy(win_main_toplev(w));
	ef_win_remove(w);
	return TRUE;
} /* win_close_common */


/*
 * Callback for "close window" menuitem and toolbar button.
 */
void
win_close_cb(GtkWidget *wgt, gpointer cbdata)
{
	(void)win_close_common(cbdata);
} /* win_close_cb */


/*
 * Callback for "delete_event" signal on main window.
 */
static int
win_main_del_event_cb(GtkWidget *wgt, GdkEvent *event, gpointer cbdata)
{
	return (!win_close_common(cbdata));
} /* win_main_del_event_cb */


/*
 * Callback for "new window" menuitem.
 */
void
win_new_cb(GtkWidget *wgt, gpointer cbdata)
{
	win_t *w;

	w = win_new();
	ef_win_add(w);
} /* win_new_cb */


static void
win_widget_failed(ef_t *efp)
{
#ifdef ENABLE_DEBUG
	g_error("Widget failed (%s, %d, %d)\n",
		efp->glname, efp->wgt_id, efp->wgt_type);
#else
	g_warning("Widget failed (%s, %d, %d)\n",
		  efp->glname, efp->wgt_id, efp->wgt_type);
#endif
} /* win_widget_failed */


void
win_widgets_setup(GladeXML *xml, win_t *w, GtkWidget **wgts, ef_t *efp)
{
	GtkWidget *wgt;

	if ((wgt = my_glade_widget_get(xml, efp->glname, 0)) == NULL)
		return;

	switch (efp->wgt_type) {
	case Ef_wgt_toplev:
	case Ef_wgt_dialog:
		if (!GTK_IS_WINDOW(wgt))
			win_widget_failed(efp);
		else
			gtk_widget_queue_resize(wgt);
		break;
	case Ef_wgt_button:
		if (!GTK_IS_BUTTON(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_checkbutton:
		if (!GTK_IS_CHECK_BUTTON(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_radiobutton:
		if (!GTK_IS_RADIO_BUTTON(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_entry:
		if (!GTK_IS_ENTRY(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_combo:
		if (!GTK_IS_COMBO(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_clist:
		if (!GTK_IS_CLIST(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_list:
		if (!GTK_IS_LIST(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_text:
		if (!GTK_IS_TEXT(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_vbox:
		if (!GTK_IS_VBOX(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_table:
		if (!GTK_IS_TABLE(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_frame:
		if (!GTK_IS_FRAME(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_notebook:
		if (!GTK_IS_NOTEBOOK(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_spinbutton:
		if (!GTK_IS_SPIN_BUTTON(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_optmenu:
		if (!GTK_IS_OPTION_MENU(wgt))
			win_widget_failed(efp);
#if 0
		else {
			menu_opt_items_t *mops;
			GtkWidget *menu, *mitem;
			GSList *group;
			int i;

			mops = (menu_opt_items_t *)efp->data;

			menu = gtk_menu_new();
			group = NULL;
			for (i = 0; mops[i].name; i++) {
				mitem = gtk_radio_menu_item_new_with_label(
							group, mops[i].name);
				gtk_signal_connect(GTK_OBJECT(mitem),
					   "activate",
					   GTK_SIGNAL_FUNC(mops[i].func), w);
				group = gtk_radio_menu_item_group(
						GTK_RADIO_MENU_ITEM(mitem));
				gtk_menu_append(GTK_MENU(menu), mitem);
				gtk_widget_show(mitem);
			}
			gtk_option_menu_set_menu(GTK_OPTION_MENU(wgt), menu);
		}
#endif
		break;
	case Ef_wgt_label:
		if (!GTK_IS_LABEL(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_menuitem:
		if (!GTK_IS_MENU_ITEM(wgt))
			win_widget_failed(efp);
		break;
	case Ef_wgt_toolbar:
		if (!GTK_IS_TOOLBAR(wgt))
			win_widget_failed(efp);
		else
			gtk_toolbar_set_style(GTK_TOOLBAR(wgt),
					      GTK_TOOLBAR_ICONS);
		break;
	case Ef_wgt_dont_care:
		break;
	default:
#ifdef ENABLE_DEBUG
		g_error("Unknown wgt_type (n=%s, wgt_id=%d, t=%d)\n",
			efp->glname, efp->wgt_id, efp->wgt_type);
#else
		g_warning("Unknown wgt_type (n=%s, wgt_id=%d, t=%d)\n",
			  efp->glname, efp->wgt_id, efp->wgt_type);
#endif
	}

	if (efp->cb && cb_signal_names[efp->wgt_type])
		gtk_signal_connect(GTK_OBJECT(wgt),
				   cb_signal_names[efp->wgt_type],
				   GTK_SIGNAL_FUNC(efp->cb), w);

	wgts[efp->wgt_id] = wgt;
} /* win_widgets_setup */


static win_t *
win_main_widgets_setup(GladeXML *xml)
{
	win_t *w;
	ef_t *efp;
	int num_main;

	g_assert(xml);

	num_main = (sizeof(ef_main_widgets) / sizeof(ef_main_widgets[0])) - 1;
	w = g_new0(win_t, 1);
	w->mainwgts = g_new0(GtkWidget *, num_main);
	w->advopts = AO_WPRINT;

#ifdef ENABLE_DEBUG
	g_print("win_main_widgets_setup: num = %d (max = %d)\n",
		num_main, Ef_main_MAX);
	num_main = 0;
#endif
	for (efp = ef_main_widgets; efp->glname; efp++) {
		win_widgets_setup(xml, w, w->mainwgts, efp);
#ifdef ENABLE_DEBUG
		g_assert(efp->wgt_id == num_main);
		num_main++;
#endif
	}
	return w;
} /* win_main_widgets_setup */


static gboolean
win_key_press(GtkWidget *txt, GdkEventKey *event, win_t *w)
{
	char *findtxt, *dirtxt;
	int len1, len2;

	switch (event->keyval) {
	case GDK_Return:
		g_assert(GTK_IS_ENTRY(w->mainwgts[Ef_main_find_text]));
		findtxt = gtk_entry_get_text(
				GTK_ENTRY(w->mainwgts[Ef_main_find_text]));
		if (findtxt)
			len1 = strlen(findtxt);
		else
			return FALSE;

		g_assert(GTK_IS_ENTRY(w->mainwgts[Ef_main_dir_text]));
		dirtxt = gtk_entry_get_text(
				GTK_ENTRY(w->mainwgts[Ef_main_dir_text]));
		if (dirtxt)
			len2 = strlen(dirtxt);
		else
			return FALSE;


		if ((GTK_WIDGET_HAS_FOCUS(w->mainwgts[Ef_main_find_text]) ||
		     GTK_WIDGET_HAS_FOCUS(w->mainwgts[Ef_main_dir_text])) &&
		    len1 > 0 && len2 > 0) {

			gtk_signal_emit_by_name(
				GTK_OBJECT(w->mainwgts[Ef_main_search]),
				"clicked");
			return TRUE;
		}

		break;
#if 0
	case GDK_KP_Enter:
	case GDK_ISO_Enter:
#endif
	case GDK_Escape:
		if (w->is_searching) {
			w->stop_search = TRUE;
			return TRUE;
		}
		break;
	}

	return FALSE;
} /* win_key_press */


void
win_progbar_update(win_t *w)
{
	float val;

	if (w->close_window || w->stop_search)
		return;

	val = gtk_progress_get_value(w->progbar) + 0.1;
	if (val > 100)
		val = 0;
	gtk_progress_set_value(w->progbar, val);
} /* win_progbar_update */


win_t *
win_new(void)
{
	GladeXML *xml;
	win_t *w;
	char *fname;

	xml = my_glade_xml_get(PACKAGE);
	g_assert(xml);
	w = win_main_widgets_setup(xml);
	gtk_object_destroy(GTK_OBJECT(xml));
	gtk_signal_connect(GTK_OBJECT(win_main_toplev(w)), "key_press_event",
			   GTK_SIGNAL_FUNC(win_key_press), w);
	w->msgbar = w->mainwgts[Ef_main_msgbar];
	w->progbar = gnome_appbar_get_progress(GNOME_APPBAR(w->msgbar));

	gtk_widget_show(w->mainwgts[Ef_wgt_toplev]);

	fname = g_strdup("./gnome-find.xpm");
	if (!g_file_exists(fname)) {
		g_free(fname);
		fname = g_concat_dir_and_file(DATADIR, "gnome-find.xpm");
	}
	if (g_file_exists(fname)) {
		GdkBitmap *mask;
		GdkPixmap *pixmap;
		GtkWidget *toplev;

		toplev = win_main_toplev(w);
		pixmap = gdk_pixmap_create_from_xpm(
					toplev->window,
					&mask,
					&toplev->style->bg[GTK_STATE_NORMAL],
					fname);
		if (pixmap)
			gdk_window_set_icon(toplev->window, NULL, pixmap, mask);
	}
	g_free(fname);

	/* set default options */
	search_reset(w);

	return w;
} /* win_new */


void
win_wprint(win_t *w, char *path, struct stat *sb, struct predicate *pred)
{
	results_add(w, path);
} /* win_wprint */


GtkWidget *
win_main_toplev(win_t *w)
{
	return (w->mainwgts ? w->mainwgts[Ef_main_toplev] : NULL);
} /* win_main_toplev */


/*
 * Sets sensitivity of either find entry box, or the directory entry box.
 */
void
win_fd_entry_sensitive(win_t *w, gboolean sensitive, gboolean is_dir)
{
	GtkWidget *wgt;

	wgt = is_dir ? w->mainwgts[Ef_main_dir_combo] :
		       w->mainwgts[Ef_main_find_combo];

	gtk_widget_set_sensitive(wgt, sensitive);
} /* win_fd_entry_sensitive */


/*
 * Sets the entry to "<Multiple %s entered>" or clears it.
 */
void
win_fd_entry_setclear(win_t *w, gboolean clear, gboolean is_dir)
{
	GtkWidget *wgt;
	char *txt;

	wgt = is_dir ? w->mainwgts[Ef_main_dir_text] :
		       w->mainwgts[Ef_main_find_text];

	if (clear) {
		txt = gtk_entry_get_text(GTK_ENTRY(wgt));
		gtk_editable_delete_text(GTK_EDITABLE(wgt), 0, strlen(txt));
		gtk_widget_set_sensitive(wgt, TRUE);
	} else {
		txt = g_strdup_printf(_("<Multiple %s entered>"),
				      is_dir ? _("directories") : _("files"));
		gtk_entry_set_text(GTK_ENTRY(wgt), txt);
		g_free(txt);
	}
} /* win_fd_entry_setclear */


/* the end */
